{% extends "email_layout.html" %}

{% block content %}
{% if operation == 'kill' %}
{% set header_prefix = 'Kill/Takedown' %}
{% else %}
{% set header_prefix = 'Takedown' %}
{% endif %}
<p>{{place}}:{{header_prefix}} Notice {{header_prefix}} Notice</p>
<p>**{{header_prefix}} NOTICE**  **{{header_prefix}} NOTICE**</p>

    {{body | safe}}

{% endblock %}
